
# Copyright (C) 2003  Pedro Werneck
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""

"""

class HoneyPythonError(Exception):
    pass

class SelectError(HoneyPythonError):
    pass

class ServerError(HoneyPythonError):
    pass

class SocketError(HoneyPythonError):
    """An error ocurred with a socket"""

class ConnectionError(SocketError):
    """An error ocurred with a connection"""

class ClientClosed(ConnectionError):
    """-1"""

class AddressAlreadyInUse(ConnectionError):
    """errno code: 98"""

class AddressNotAvailable(ConnectionError):
    """errno code: 99"""

class NetworkIsDown(ConnectionError):
    """errno code: 100"""

class NetworkUnreachable(ConnectionError):
    """errno code: 101"""

class NetworkReset(ConnectionError):
    """errno code: 102"""

class ConnectionAborted(ConnectionError):
    """errno code: 103"""

class ConnectionResetByPeer(ConnectionError):
    """errno code: 104"""

class NoBufferSpaceAvailable(ConnectionError):
    """errno code: 105"""

class SocketIsConnected(ConnectionError):
    """errno code: 106"""

class NotConnected(ConnectionError):
    """errno code: 107"""

class TCPTimeOut(ConnectionError):
    """errno code: 110"""

class ConnectionRefused(ConnectionError):
    """errno code: 111"""

class HostIsDown(ConnectionError):
    """errno code: 112"""

class HostUnreachable(ConnectionError):
    """errno code: 113"""

class ConnectionAlreadyInProgress(ConnectionError):
    """errno code: 114"""

_connErrorMap = {-1:ClientClosed,
                 98:AddressAlreadyInUse,
                 99:AddressNotAvailable,
                 100:NetworkIsDown,
                 101:NetworkUnreachable,
                 102:NetworkReset,
                 103:ConnectionAborted,
                 104:ConnectionResetByPeer,
                 105:NoBufferSpaceAvailable,
                 106:SocketIsConnected,
                 107:NotConnected,
                 110:TCPTimeOut,
                 111:ConnectionRefused,
                 112:HostIsDown,
                 113:HostUnreachable,
                 114:ConnectionAlreadyInProgress}


class MapSocketError(ConnectionError):
    """A especial exception that maps errno error codes to
    specific ConnectionError exceptions for each case.

    """
    # this could be easily done with __new__, but since exceptions are
    # still old style classes, this ugly hack is needed
    def __init__(self, reason):
        """
        @param reason: the (code, msg) tuple returned by a
            socket.error exception
            
        """
        code, msg = reason
        exc = _connErrorMap.get(code, ConnectionError)
        self.__class__ = exc
        ConnectionError.__init__(self)
