
# Copyright (C) 2003  Pedro Werneck
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""

"""

# System imports
import socket

# Exceptions
from Honey.Error import ServerError, ConnectionError, SelectError
from Honey.Error import AddressAlreadyInUse

# HoneyPython imports
from Honey.Protocol import BaseProtocol
from Honey.Socket import ServerSocket
from Honey.Loop import Engine
from Honey.Selector import Select, READ, WRITE


class TCPServer(object):
    def __init__(self, host=None, type=None, engine=None, select=None):
        self.host = host or socket.gethostname()
        self.type = type or ServerSocket
        self.engine = engine or Engine()
        self.select = select or Select()
        self.ports = {}

    def clean(self):
        for socket in self.ports.values():
            socket.close()
        self.ports.clear()
        self.select.clean()
        
    def bind(self, port, protocol):
        if not issubclass(protocol, BaseProtocol):
            raise ServerError("protocol is not valid")

        if port in self.ports:
            name = self.ports[port].protocol.__name__
            raise ServerError("port %d already bound with %s"%(port, name))
        
        self.ports[port] = socket = ServerSocket(self, protocol)
        socket.bind(self.host, port)
        socket.listen()
        self.select.register(socket, READ)

    def _accept(self, sock):
        try:
            client = sock.accept()
            self.select.register(client, READ)
            self.select.register(client, WRITE)
        except SelectError:
            pass

    def _read(self, client):
        try:
            client.recv()
        except ConnectionError, reason:
            self._close(client, reason)
        
    def _write(self, client):
        try:
            client.flush()
        except ConnectionError, reason:
            self._close(client, reason)

    def _close(self, client, reason=None):
        # This will assert that any data on the client is flushed
        # before closing
        if client._buffer:
            self.engine.register(0.001, self._close, client, reason=reason)
            return
        # I believe there's no risk of recursive calling because an
        # unhealthy socket (not flushing) will be pruned and closed
        # anyway... but I may be wrong
        try:
            try:
                self.select.unregister(client, READ)
                self.select.unregister(client, WRITE)
            except SelectError:
                pass
        finally:
            client._close(reason)

    def _update(self):
        r, w = self.select.doSelect()

        for sock in r:
            if isinstance(sock, ServerSocket):
                self._accept(sock)
            else:
                self._read(sock)

        for sock in w:
            if sock._buffer:
                self._write(sock)

        self._id = self.engine.register(0, self._update)

    def stop(self):
        self.engine.unregister(self._id)
        self.engine.stop()

    def run(self):
        self._id = self.engine.register(0, self._update)
        self.engine.run()
        

