
# Copyright (C) 2003  Pedro Werneck
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""

"""
# System imports

import socket
import weakref
from errno import EWOULDBLOCK


# HoneyPython imports

from Honey.Error import MapSocketError


class BaseSocket(object):

    family = socket.AF_INET
    type = socket.SOCK_STREAM

    def __init__(self, server, sock=None):
        self._server = weakref.ref(server)
        self.socket = sock or socket.socket(self.family, self.type)
        self.socket.setblocking(0)
        self.fileno = self.socket.fileno
        self.closed = False
        self.attrs = {}

    # Ugly, ugly, ugly hack... I don't want to create reference cycles
    # with the server, and it will outlive clients... of course, I
    # don't need property for this, but I don't want to need to
    # remember that this is a weakref
    server = property(lambda self: self._server())

    def __nonzero__(self):
        return not self.closed

    def close(self, reason=None):
        self._close()
    
    def _close(self, reason=None):
        try:
            try:
                self.socket.shutdown(2)
            except socket.error:
                self.socket.close()
        finally:
            self.closed = True


class ClientSocket(BaseSocket):
    bufsize = 8192

    def __init__(self, server, sock, addr):
        super(ClientSocket, self).__init__(server, sock)
        self.addr = addr
        self.attrs['host'], self.attrs['port'] = addr
        self.attrs['server'] = server.host
        self._buffer = ""

        self.connectionEstablished()

    def recv(self):
        try:
            data = self.socket.recv(self.bufsize)
        except socket.error, reason:
            if reason.args[0] == EWOULDBLOCK:
                return
            else:
                raise MapSocketError(reason.args)
        if not data:
            raise MapSocketError(('-1', 'Other side has closed connection'))

        self.dataReceived(data)

    def send(self, data):
        self._buffer += data

    def flush(self):
        try:
            sbytes = self.socket.send(self._buffer)
            sdata = self._buffer[:sbytes]
            self._buffer = self._buffer[sbytes:]
        except socket.error, reason:
            if reason.args[0] == EWOULDBLOCK:
                return ''
            else:
                raise MapSocketError(reason.args)
            
        return sdata
    
    def close(self, reason=None):
        self.server._close(self)

    def _close(self, reason=None):
        self.connectionLost()
        super(ClientSocket, self)._close(reason)


class ServerSocket(BaseSocket):
    def __init__(self, server, protocol):
        super(ServerSocket, self).__init__(server)
        self.addr = None
        self.name = name = protocol.__name__
        self.protocol = type(name, (protocol, ClientSocket), {})
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

    def bind(self, host, port):
        try:
            self.socket.bind((host, port))
            self.addr = (host, port)
        except socket.error, reason:
            raise MapSocketError(reason.args)

    def listen(self, backlog=1):
        try:
            self.socket.listen(backlog)
        except socket.error, reason:
            raise MapSocketError(reason.args)

    def accept(self):
        try:
            sock, addr = self.socket.accept()
            return self.protocol(self.server, sock, addr)
        except socket.error, reason:
            raise MapSocketError(reason.args)
            
        

            
